(function () {
    'use strict';

    var url             = require('url'),
        EventEmitter    = require('events').EventEmitter,
        extend          = require('util')._extend,
        http            = require('http');

    /**
     * Helpers
     */
    // var getPort = function(req) {
    //     return  (req.headers.host ? req.headers.host.match(/:(\d+)/) : []).pop() ||
    //             (req.connection.pair ? '443' : '80');
    // };

    var setupOutgoing = function(outgoing, options, req, forward) {
        outgoing.port = options.target.port ||
            (~['https:', 'wss:'].indexOf(options.target.protocol) ? 443 : 80);

        ['host', 'hostname', 'socketPath'].forEach(function (e) {
            outgoing[e] = options.target[e];
        });

        ['method', 'headers'].forEach(function (e) {
            outgoing[e] = req[e];
        });

        if (options.headers) {
            extend(outgoing.headers, options.headers);
        }

        if (options.target.protocol === 'https:') {
            outgoing.rejectUnauthorized = (typeof options.secure === "undefined") ? true : options.secure;
        }

        outgoing.agent = options.agent || false;

        // ================ 1.0.2
        outgoing.path = url.parse(req.url).path;
        return outgoing;

      // // ================ Master
      // outgoing.localAddress = options.localAddress;

      // //
      // // Remark: If we are false set the connection: close. This is the right thing to do
      // // as node core doesn't handle this COMPLETELY properly yet.
      // //
      // if(!outgoing.agent) {
      //   outgoing.headers = outgoing.headers || {};
      //   outgoing.headers.connection = 'close';
      // }

      // //
      // // Remark: Can we somehow not use url.parse as a perf optimization?
      // //
      // outgoing.path = !options.toProxy
      //   ? url.parse(req.url).path
      //   : req.url;

      // return outgoing;
    };


    var ProxyServer = function (options) {
        // ['target', 'forward'].forEach(function(e) {
        //     if (typeof options[e] === 'string') {
        //         options[e] = url.parse(options[e]);
        //     }
        // });

        // var self = this;
        this.web = function (req, res, opts) {
            // Merge in global options
            Object.keys(options).forEach(function (key) {
                if (key in opts) {
                    return;
                }
                opts[key] = options[key];
            });

            // Request initalization
            var proxyReq = http.request(
                setupOutgoing(opts.ssl || {}, opts, req)
            );

            req.pipe(proxyReq);

            proxyReq.on('response', function(proxyRes) {
                console.warn('responding');
                Object.keys(proxyRes.headers).forEach(function(key) {
                    res.setHeader(key, proxyRes.headers[key]);
                });
                res.writeHead(proxyRes.statusCode);
                proxyRes.pipe(res);
            });
        };
    };

    require('util').inherits(ProxyServer, EventEmitter);


    var createProxyServer = function (options) {
        return new ProxyServer(options);
    }

    module.exports = {
        createProxyServer: createProxyServer
    };

})();












// XXX backport of http-proxy
// (function () {
//     'use strict';

//     var url = require('url'),
//         EventEmitter = require('events').EventEmitter,
//         extend = require('util')._extend,
//         http      = require('http'),
//         https     = require('https');

//     /**
//      * Helpers
//      */

//     var getPort = function(req) {
//         return  (req.headers.host ? req.headers.host.match(/:(\d+)/) : []).pop() ||
//                 (req.connection.pair ? '443' : '80');
//     };

//     var setupOutgoing = function(outgoing, options, req, forward) {
//         outgoing.port = options[forward || 'target'].port ||
//             (~['https:', 'wss:'].indexOf(options[forward || 'target'].protocol) ? 443 : 80);

//         ['host', 'hostname', 'socketPath'].forEach(function (e) {
//             outgoing[e] = options[forward || 'target'][e];
//         });

//         ['method', 'headers'].forEach(function (e) {
//             outgoing[e] = req[e];
//         });

//         if (options.headers) {
//             extend(outgoing.headers, options.headers);
//         }

//         if (options[forward || 'target'].protocol === 'https:') {
//             outgoing.rejectUnauthorized = (typeof options.secure === "undefined") ? true : options.secure;
//         }

//         outgoing.agent = options.agent || false;

//         // ================ 1.0.2
//         outgoing.path = url.parse(req.url).path;
//         return outgoing;

//       // // ================ Master
//       // outgoing.localAddress = options.localAddress;

//       // //
//       // // Remark: If we are false set the connection: close. This is the right thing to do
//       // // as node core doesn't handle this COMPLETELY properly yet.
//       // //
//       // if(!outgoing.agent) {
//       //   outgoing.headers = outgoing.headers || {};
//       //   outgoing.headers.connection = 'close';
//       // }

//       // //
//       // // Remark: Can we somehow not use url.parse as a perf optimization?
//       // //
//       // outgoing.path = !options.toProxy
//       //   ? url.parse(req.url).path
//       //   : req.url;

//       // return outgoing;
//     };


//     /**
//      * Incoming
//      */

//     var httpPass = function (req, res, options, server) {
//         if (req.method === 'DELETE' && !req.headers['content-length']) {
//             req.headers['content-length'] = '0';
//         }

//         if (options.timeout) {
//             req.socket.setTimeout(options.timeout);
//         }

//         if (options.xfwd) {
//             var values = {
//                 for  : req.connection.remoteAddress || req.socket.remoteAddress,
//                 port : getPort(req),
//                 proto: req.isSpdy ? 'https' : (req.connection.pair ? 'https' : 'http')
//             };

//             ['for', 'port', 'proto'].forEach(function(header) {
//                 req.headers['x-forwarded-' + header] =
//                     (req.headers['x-forwarded-' + header] || '') +
//                     (req.headers['x-forwarded-' + header] ? ',' : '') +
//                     values[header];
//             });
//         }

//         // Actual proxying

//         if (options.forward) {
//             // If forward enable, so just pipe the request
//             var forwardReq = (options.forward.protocol === 'https:' ? https : http).request(
//                 setupOutgoing(options.ssl || {}, options, req, 'forward')
//             );
//             (options.buffer || req).pipe(forwardReq);
//             if(!options.target) {
//                 return res.end();
//             }
//         }

//         // Request initalization
//         var proxyReq = (options.target.protocol === 'https:' ? https : http).request(
//             setupOutgoing(options.ssl || {}, options, req)
//         );

//         // Error Handler
//         proxyReq.on('error', function(err){
//             if(options.buffer) {
//                 options.buffer.destroy();
//             }
//             server.emit('error', err, req, res);
//         });

//         (options.buffer || req).pipe(proxyReq);

//         proxyReq.on('response', function(proxyRes) {
//             if(server) {
//                 server.emit('proxyRes', proxyRes);
//             }

//             if(req.httpVersion === '1.0') {
//                 delete proxyRes.headers['transfer-encoding'];
//             }

//             if (req.httpVersion === '1.0') {
//                 proxyRes.headers.connection = req.headers.connection || 'close';
//             } else if (!proxyRes.headers.connection) {
//                 proxyRes.headers.connection = req.headers.connection || 'keep-alive';
//             }

//             Object.keys(proxyRes.headers).forEach(function(key) {
//                 res.setHeader(key, proxyRes.headers[key]);
//             });

//             res.writeHead(proxyRes.statusCode);


//             proxyRes.pipe(res);
//         });
//     };

//     var ProxyServer = function (options) {
//         EventEmitter.call(this);

//         ['target', 'forward'].forEach(function(e) {
//             if (typeof options[e] === 'string') {
//                 options[e] = url.parse(options[e]);
//             }
//         });

//         var self = this;
//         this.web = function (req, res, opts) {
//             Object.keys(options).forEach(function (key) {
//                 if (key in opts) {
//                     return;
//                 }
//                 opts[key] = options[key];
//             });
//             httpPass(req, res, opts, self);
//         };
//     };

//     require('util').inherits(ProxyServer, EventEmitter);


//     var createProxyServer = function (options) {
//         return new ProxyServer(options);
//     }

//     module.exports = {
//         createProxyServer: createProxyServer
//     };

// })();
